package com.emdoor.scanbcdemo;

import androidx.appcompat.app.AppCompatActivity;

import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.ToggleButton;

public class MainActivity extends AppCompatActivity {
    // Receive QR code and bar code data action and extra
    public static final String BARCODEPORT_RECEIVEDDATA_ACTION = "com.android.serial.BARCODEPORT_RECEIVEDDATA_ACTION";
    public static final String BARCODEPORT_RECEIVEDDATA_EXTRA_DATA = "DATA";

    // Simulate scanning keys
    static public final String ACTION_KEYEVENT_KEYCODE_SCAN_L_DOWN = "com.android.action.keyevent.KEYCODE_KEYCODE_SCAN_L_DOWN";
    static public final String ACTION_KEYEVENT_KEYCODE_SCAN_L_UP = "com.android.action.keyevent.KEYCODE_KEYCODE_SCAN_L_UP";
    static public final String ACTION_KEYEVENT_KEYCODE_SCAN_R_DOWN = "com.android.action.keyevent.KEYCODE_KEYCODE_SCAN_R_DOWN";
    static public final String ACTION_KEYEVENT_KEYCODE_SCAN_R_UP = "com.android.action.keyevent.KEYCODE_KEYCODE_SCAN_R_UP";
    public static final String ACTION_KEYEVENT_SCAN_F_UP = "com.android.action.keyevent.KEYCODE_KEYCODE_SCAN_F_UP";
    public static final String ACTION_KEYEVENT_SCAN_F_DOWN = "com.android.action.keyevent.KEYCODE_KEYCODE_SCAN_F_DOWN";

    // service for 4710
    static final String PACKAGE_NAME = "com.zebra.scanner";
    static final String SERVICE_NAME = "com.zebra.scanner.ScannerJsbService";
    // service for 6703
//    static final String PACKAGE_NAME = "com.emdoor.scan6703";
//    static final String SERVICE_NAME = "com.emdoor.scan6703.scanner.ScannerService";

    // switch scanner enable state
    static final String ACTION_CHANGE_STATE_ENABLE = "com.zebra.action.CHANGE_STATE_ENABLE";
    static final String ACTION_CHANGE_STATE_DISABLE = "com.zebra.action.CHANGE_STATE_DISABLE";

    private TextView tvShow;
    private Button bEnable;
    private Button bDisable;
    private ToggleButton bScan;

    private Handler handler = new Handler();

    BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            bScan.setChecked(false);
            String data = intent.getStringExtra(BARCODEPORT_RECEIVEDDATA_EXTRA_DATA);   // get data from intent
            if (data != null && data.length() > 0) {
                tvShow.append(data + "\r\n");          // show data
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        tvShow = findViewById(R.id.tv_show);
        bEnable = findViewById(R.id.b_enable);
        bDisable = findViewById(R.id.b_disable);
        bScan = findViewById(R.id.b_scan);

        bEnable.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                bScan.setChecked(false);
                bEnable.setEnabled(false);                                                  // disable buttons and restore them later to prevent
                bDisable.setEnabled(false);                                                 // the possible error of switching states too fast
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        bEnable.setEnabled(true);                                           // restore buttons
                        bDisable.setEnabled(true);
                    }
                }, 1500);
                Intent intent = new Intent();
                intent.setComponent(new ComponentName(PACKAGE_NAME, SERVICE_NAME));
                startService(intent);                                                       // start service
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        Intent intent = new Intent(ACTION_CHANGE_STATE_ENABLE);
                        sendBroadcast(intent);                                              // then enable scanner
                    }
                }, 300);
            }
        });

        bDisable.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                bScan.setChecked(false);
                bEnable.setEnabled(false);                                                  // disable buttons
                bDisable.setEnabled(false);
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        bEnable.setEnabled(true);                                           // restore buttons
                        bDisable.setEnabled(true);
                    }
                }, 1500);
                Intent intent = new Intent(ACTION_CHANGE_STATE_DISABLE);
                sendBroadcast(intent);                                                      // disable scanner
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        Intent intent = new Intent();
                        intent.setComponent(new ComponentName(PACKAGE_NAME, SERVICE_NAME));
                        stopService(intent);                                                // then stop service
                    }
                }, 300);
            }
        });

        bScan.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String action = bScan.isChecked() ?
                        ACTION_KEYEVENT_KEYCODE_SCAN_R_DOWN :                               // trigger scan action
                        ACTION_KEYEVENT_KEYCODE_SCAN_R_UP;                                  // stop scan action
                Intent intent = new Intent(action);
                sendBroadcast(intent);
            }
        });
        // Register QR code and bar code data receiver.
        IntentFilter filter = new IntentFilter(BARCODEPORT_RECEIVEDDATA_ACTION);
        registerReceiver(receiver, filter);
    }

    @Override
    protected void onDestroy() {
        unregisterReceiver(receiver);
        super.onDestroy();
    }
}
